@@ -1,8 +1,8 @@
 /* Copyright 2003-2004 Roger Dingledine
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: compat.c,v 1.63 2005/08/03 17:16:48 nickm Exp $ */
-const char compat_c_id[] = "$Id: compat.c,v 1.63 2005/08/03 17:16:48 nickm Exp $";
+/* $Id: compat.c,v 1.66 2005/08/26 18:40:44 arma Exp $ */
+const char compat_c_id[] = "$Id: compat.c,v 1.66 2005/08/26 18:40:44 arma Exp $";
 
 /**
  * \file compat.c
@@ -161,7 +161,7 @@ tor_memmem(const void *_haystack, size_t hlen, const void *_needle, size_t nlen)
   end = haystack + hlen;
   first = *(const char*)needle;
   while ((p = memchr(p, first, end-p))) {
-    if (p+nlen >= end)
+    if (p+nlen > end)
       return NULL;
     if (!memcmp(p, needle, nlen))
       return p;
@@ -533,7 +533,7 @@ get_user_homedir(const char *username)
   tor_assert(username);
 
   if (!(pw = getpwnam(username))) {
-    log_fn(LOG_ERR,"User '%s' not found.", username);
+    log_fn(LOG_ERR,"User \"%s\" not found.", username);
     return NULL;
   }
   return tor_strdup(pw->pw_dir);
@@ -579,6 +579,7 @@ tor_lookup_hostname(const char *name, uint32_t *addr)
    * something.
    */
   struct in_addr iaddr;
+  tor_assert(name);
   tor_assert(addr);
   if (!*name) {
     /* Empty address is an error. */
