@@ -1,8 +1,8 @@
 /* Copyright 2001,2002,2003 Roger Dingledine, Matej Pfajfar.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: log.c,v 1.97 2005/08/08 17:31:57 arma Exp $ */
-const char log_c_id[] = "$Id: log.c,v 1.97 2005/08/08 17:31:57 arma Exp $";
+/* $Id: log.c,v 1.98 2005/08/12 17:24:53 nickm Exp $ */
+const char log_c_id[] = "$Id: log.c,v 1.98 2005/08/12 17:24:53 nickm Exp $";
 
 /**
  * \file log.c
@@ -14,6 +14,12 @@ const char log_c_id[] = "$Id: log.c,v 1.97 2005/08/08 17:31:57 arma Exp $";
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h>
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
+#ifdef HAVE_TIME_H
+#include <time.h>
+#endif
 #include "./util.h"
 #include "./log.h"
 
