@@ -1,7 +1,7 @@
 /* Copyright 2003-2004 Roger Dingledine
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: util.h,v 1.138 2005/09/08 18:33:51 nickm Exp $ */
+/* $Id: util.h,v 1.139 2005/09/13 06:19:31 nickm Exp $ */
 
 /**
  * \file util.h
@@ -10,7 +10,7 @@
 
 #ifndef __UTIL_H
 #define __UTIL_H
-#define UTIL_H_ID "$Id: util.h,v 1.138 2005/09/08 18:33:51 nickm Exp $"
+#define UTIL_H_ID "$Id: util.h,v 1.139 2005/09/13 06:19:31 nickm Exp $"
 
 #include "orconfig.h"
 #include "torint.h"
@@ -121,6 +121,16 @@ int check_private_dir(const char *dirname, cpd_check_t check);
 int write_str_to_file(const char *fname, const char *str, int bin);
 int write_bytes_to_file(const char *fname, const char *str, size_t len,
                         int bin);
+typedef struct sized_chunk_t {
+  const char *bytes;
+  size_t len;
+} sized_chunk_t;
+struct smartlist_t;
+int write_chunks_to_file(const char *fname, const struct smartlist_t *chunks,
+                         int bin);
+int append_bytes_to_file(const char *fname, const char *str, size_t len,
+                         int bin);
+
 char *read_file_to_str(const char *filename, int bin);
 char *parse_line_from_str(char *line, char **key_out, char **value_out);
 char *expand_filename(const char *filename);
