@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: connection_edge.c,v 1.345 2005/09/03 02:23:53 nickm Exp $ */
-const char connection_edge_c_id[] = "$Id: connection_edge.c,v 1.345 2005/09/03 02:23:53 nickm Exp $";
+/* $Id: connection_edge.c,v 1.348 2005/09/12 22:05:17 arma Exp $ */
+const char connection_edge_c_id[] = "$Id: connection_edge.c,v 1.348 2005/09/12 22:05:17 arma Exp $";
 
 /**
  * \file connection_edge.c
@@ -173,7 +173,7 @@ connection_edge_destroy(uint16_t circ_id, connection_t *conn)
 int
 connection_edge_end(connection_t *conn, char reason, crypt_path_t *cpath_layer)
 {
-  char payload[5];
+  char payload[RELAY_PAYLOAD_SIZE];
   size_t payload_len=1;
   circuit_t *circ;
 
@@ -190,9 +190,8 @@ connection_edge_end(connection_t *conn, char reason, crypt_path_t *cpath_layer)
   }
 
   payload[0] = reason;
-  if (reason == END_STREAM_REASON_EXITPOLICY) {
-    /* this is safe even for rend circs, because they never fail
-     * because of exitpolicy */
+  if (reason == END_STREAM_REASON_EXITPOLICY &&
+      !connection_edge_is_rendezvous_stream(conn)) {
     set_uint32(payload+1, htonl(conn->addr));
     set_uint32(payload+5, htonl(MAX_DNS_ENTRY_AGE)); /* XXXXfill with a real TTL*/
     payload_len += 8;
