@@ -1,7 +1,7 @@
 /* Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: control.c,v 1.127 2005/09/08 06:37:50 nickm Exp $ */
-const char control_c_id[] = "$Id: control.c,v 1.127 2005/09/08 06:37:50 nickm Exp $";
+/* $Id: control.c,v 1.130 2005/09/14 02:36:29 nickm Exp $ */
+const char control_c_id[] = "$Id: control.c,v 1.130 2005/09/14 02:36:29 nickm Exp $";
 
 /**
  * \file control.c
@@ -599,16 +599,17 @@ get_stream(const char *id)
   if (!ok)
     return NULL;
   conn = connection_get_by_global_id(n_id);
-  if (conn->type != CONN_TYPE_AP)
+  if (!conn || conn->type != CONN_TYPE_AP)
     return NULL;
   return conn;
 }
 
 /** Helper for setconf and resetconf. Acts like setconf, except
- * it passes <b>reset</b> on to options_trial_assign().
+ * it passes <b>use_defaults</b> on to options_trial_assign().
  */
 static int
-control_setconf_helper(connection_t *conn, uint32_t len, char *body, int reset)
+control_setconf_helper(connection_t *conn, uint32_t len, char *body,
+                       int use_defaults, int clear_first)
 {
   int r;
   config_line_t *lines=NULL;
@@ -666,19 +667,33 @@ control_setconf_helper(connection_t *conn, uint32_t len, char *body, int reset)
     }
   }
 
-  if ((r=options_trial_assign(lines, reset)) < 0) {
+  if ((r=options_trial_assign(lines, use_defaults, clear_first)) < 0) {
+    int v0_err;
+    const char *msg;
     log_fn(LOG_WARN,"Controller gave us config lines that didn't validate.");
-    if (r==-1) {
-      if (v0)
-        send_control0_error(conn, ERR_UNRECOGNIZED_CONFIG_KEY,
-                            "Unrecognized option");
-      else
-        connection_write_str_to_buf("552 Unrecognized option\r\n", conn);
+    switch (r) {
+      case -1:
+        v0_err = ERR_UNRECOGNIZED_CONFIG_KEY;
+        msg = "Unrecognized option";
+        break;
+      case -2:
+        v0_err = ERR_INVALID_CONFIG_VALUE;
+        msg = "Unrecognized option value";
+        break;
+      case -3:
+        v0_err = ERR_INVALID_CONFIG_VALUE;
+        msg = "Transition not allowed";
+        break;
+      case -4:
+      default:
+        v0_err = ERR_INVALID_CONFIG_VALUE;
+        msg = "Unable to set option";
+        break;
+    }
+    if (v0) {
+      send_control0_error(conn, v0_err, msg);
     } else {
-      if (v0)
-        send_control0_error(conn,ERR_INVALID_CONFIG_VALUE,"Invalid option value");
-      else
-        connection_write_str_to_buf("552 Invalid option value\r\n", conn);
+      connection_printf_to_buf(conn, "552 %s\r\n", msg);
     }
     config_free_lines(lines);
     return 0;
@@ -693,7 +708,7 @@ control_setconf_helper(connection_t *conn, uint32_t len, char *body, int reset)
 static int
 handle_control_setconf(connection_t *conn, uint32_t len, char *body)
 {
-  return control_setconf_helper(conn, len, body, 0);
+  return control_setconf_helper(conn, len, body, 0, 1);
 }
 
 /** Called when we receive a RESETCONF message: parse the body and try
@@ -703,7 +718,7 @@ handle_control_resetconf(connection_t *conn, uint32_t len, char *body)
 {
   int v0 = STATE_IS_V0(conn->state);
   tor_assert(!v0);
-  return control_setconf_helper(conn, len, body, 1);
+  return control_setconf_helper(conn, len, body, 1, 1);
 }
 
 /** Called when we receive a GETCONF message.  Parse the request, and
