@@ -1,8 +1,8 @@
 /* Copyright 2003-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: cpuworker.c,v 1.84 2005/06/30 07:17:38 nickm Exp $ */
-const char cpuworker_c_id[] = "$Id: cpuworker.c,v 1.84 2005/06/30 07:17:38 nickm Exp $";
+/* $Id: cpuworker.c,v 1.86 2005/08/15 23:46:18 arma Exp $ */
+const char cpuworker_c_id[] = "$Id: cpuworker.c,v 1.86 2005/08/15 23:46:18 arma Exp $";
 
 /**
  * \file cpuworker.c
@@ -46,8 +46,7 @@ static void process_pending_task(connection_t *cpuworker);
 void
 cpu_init(void)
 {
-  last_rotation_time=time(NULL);
-  spawn_enough_cpuworkers();
+  cpuworkers_rotate();
 }
 
 /** Called when we're done sending a request to a cpuworker. */
@@ -101,7 +100,8 @@ cpuworkers_rotate(void)
     --num_cpuworkers;
   }
   last_rotation_time = time(NULL);
-  spawn_enough_cpuworkers();
+  if (server_mode(get_options()))
+    spawn_enough_cpuworkers();
 }
 
 /** If the cpuworker closes the connection,
