@@ -1,8 +1,8 @@
 /* Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: directory.c,v 1.270 2005/09/09 19:37:12 nickm Exp $ */
-const char directory_c_id[] = "$Id: directory.c,v 1.270 2005/09/09 19:37:12 nickm Exp $";
+/* $Id: directory.c,v 1.275 2005/09/14 03:49:17 arma Exp $ */
+const char directory_c_id[] = "$Id: directory.c,v 1.275 2005/09/14 03:49:17 arma Exp $";
 
 #include "or.h"
 
@@ -51,6 +51,8 @@ static int body_is_plausible(const char *body, size_t body_len, int purpose);
 static int purpose_is_private(uint8_t purpose);
 static char *http_get_header(const char *headers, const char *which);
 static char *http_get_origin(const char *headers, connection_t *conn);
+static void connection_dir_download_networkstatus_failed(connection_t *conn);
+static void dir_networkstatus_download_failed(smartlist_t *failed);
 
 /********* START VARIABLES **********/
 
@@ -261,19 +263,54 @@ directory_initiate_command_trusted_dir(trusted_dir_server_t *dirserv,
                payload, payload_len);
 }
 
-/** Called when we are unable to complete our connection to a
+/** Called when we are unable to complete the client's request to a
  * directory server: Mark the router as down and try again if possible.
  */
 void
-connection_dir_connect_failed(connection_t *conn)
+connection_dir_request_failed(connection_t *conn)
 {
+  if (router_digest_is_me(conn->identity_digest))
+    return; /* this was a test fetch. don't retry. */
   router_mark_as_down(conn->identity_digest); /* don't try him again */
   if (conn->purpose == DIR_PURPOSE_FETCH_DIR ||
       conn->purpose == DIR_PURPOSE_FETCH_RUNNING_LIST) {
-    log_fn(LOG_INFO, "Giving up on directory server at '%s'; retrying",
-           conn->address);
+    log_fn(LOG_INFO, "Giving up on directory server at '%s:%d'; retrying",
+           conn->address, conn->port);
     directory_get_from_dirserver(conn->purpose, NULL,
                                  0 /* don't retry_if_no_servers */);
+  } else if (conn->purpose == DIR_PURPOSE_FETCH_NETWORKSTATUS) {
+    log_fn(LOG_INFO, "Giving up on directory server at '%s'; retrying",
+           conn->address);
+    connection_dir_download_networkstatus_failed(conn);
+  }
+}
+
+/** Called when an attempt to download one or network status documents
+ * on connection <b>conn</b> failed.
+ */
+static void
+connection_dir_download_networkstatus_failed(connection_t *conn)
+{
+  if (!strcmpstart(conn->requested_resource, "all")) {
+    /* We're a non-authoritative directory cache; try again. */
+    directory_get_from_dirserver(conn->purpose, "all.z",
+                                 0 /* don't retry_if_no_servers */);
+  } else if (!strcmpstart(conn->requested_resource, "fp/")) {
+    /* We were trying to download by fingerprint; mark them all has having
+     * failed, and possibly retry them later.*/
+    smartlist_t *failed = smartlist_create();
+    /* XXXX NM this splitting logic is duplicated someplace. Fix that. */
+    smartlist_split_string(failed, conn->requested_resource+3, "+", 0, 0);
+    if (smartlist_len(failed)) {
+      char *last = smartlist_get(failed,smartlist_len(failed)-1);
+      size_t last_len = strlen(last);
+      if (!strcmp(last+last_len-2, ".z"))
+        last[last_len-2] = '\0';
+
+      dir_networkstatus_download_failed(failed);
+      SMARTLIST_FOREACH(failed, char *, cp, tor_free(cp));
+    }
+    smartlist_free(failed);
   }
 }
 
@@ -346,7 +383,7 @@ directory_initiate_command(const char *address, uint32_t addr,
 
     switch (connection_connect(conn, conn->address, addr, dir_port)) {
       case -1:
-        connection_dir_connect_failed(conn);
+        connection_dir_request_failed(conn); /* retry if we want */
         connection_free(conn);
         return;
       case 1:
@@ -728,8 +765,11 @@ body_is_plausible(const char *body, size_t len, int purpose)
 /** We are a client, and we've finished reading the server's
  * response. Parse and it and act appropriately.
  *
- * Return -1 if an error has occurred, or 0 normally. The caller
- * will take care of marking the connection for close.
+ * If we're happy with the result (we get it and it's useful),
+ * return 0. Otherwise return -1, and the caller should consider
+ * trying the request again.
+ *
+ * The caller will take care of marking the connection for close.
  */
 static int
 connection_dir_client_reached_eof(connection_t *conn)
@@ -844,7 +884,7 @@ connection_dir_client_reached_eof(connection_t *conn)
       log_fn(LOG_INFO,"Empty directory; status %d (\"%s\") Ignoring.",
              status_code, reason);
       tor_free(body); tor_free(headers); tor_free(reason);
-      return 0;
+      return -1;
     }
     if (status_code != 200) {
       log_fn(LOG_WARN,"Received http status code %d (\"%s\") from server '%s:%d'. I'll try again soon.",
@@ -854,9 +894,10 @@ connection_dir_client_reached_eof(connection_t *conn)
     }
     if (router_load_routerlist_from_directory(body, NULL, !skewed, 0) < 0) {
       log_fn(LOG_NOTICE,"I failed to parse the directory I fetched from '%s:%d'. Ignoring.", conn->address, conn->port);
-    } else {
-      log_fn(LOG_INFO,"updated routers.");
+      tor_free(body); tor_free(headers); tor_free(reason);
+      return -1;
     }
+    log_fn(LOG_INFO,"updated routers.");
     /* do things we've been waiting to do */
     directory_has_arrived(time(NULL), conn->identity_digest);
   }
@@ -896,6 +937,7 @@ connection_dir_client_reached_eof(connection_t *conn)
              status_code, reason, conn->address, conn->port,
              conn->requested_resource);
       tor_free(body); tor_free(headers); tor_free(reason);
+      connection_dir_download_networkstatus_failed(conn);
       return -1;
     }
     if (conn->requested_resource &&
@@ -922,6 +964,9 @@ connection_dir_client_reached_eof(connection_t *conn)
         break;
     }
     if (which) {
+      if (smartlist_len(which)) {
+        dir_networkstatus_download_failed(which);
+      }
       SMARTLIST_FOREACH(which, char *, cp, tor_free(cp));
       smartlist_free(which);
     }
@@ -938,7 +983,7 @@ connection_dir_client_reached_eof(connection_t *conn)
         log_fn(LOG_INFO,"eof (status 200) after uploading server descriptor: finished.");
         break;
       case 400:
-        log_fn(LOG_WARN,"http status 400 (\"%s\") response from dirserver '%s:%d'. Malformed server descriptor?", reason, conn->address, conn->port);
+        log_fn(LOG_WARN,"http status 400 (\"%s\") response from dirserver '%s:%d'. Please correct.", reason, conn->address, conn->port);
         break;
       case 403:
         log_fn(LOG_WARN,"http status 403 (\"%s\") response from dirserver '%s:%d'. Is your clock skewed? Have you mailed us your key fingerprint? Are you using the right key? Are you using a private IP address? See http://tor.eff.org/doc/tor-doc-server.html", reason, conn->address, conn->port);
@@ -947,6 +992,8 @@ connection_dir_client_reached_eof(connection_t *conn)
         log_fn(LOG_WARN,"http status %d (\"%s\") reason unexpected (server '%s:%d').", status_code, reason, conn->address, conn->port);
         break;
     }
+    /* return 0 in all cases, since we don't want to mark any
+     * dirservers down just because they don't like us. */
   }
 
   if (conn->purpose == DIR_PURPOSE_FETCH_RENDDESC) {
@@ -1007,6 +1054,8 @@ connection_dir_reached_eof(connection_t *conn)
   }
 
   retval = connection_dir_client_reached_eof(conn);
+  if (retval == 0) /* success */
+    conn->state = DIR_CONN_STATE_CLIENT_FINISHED;
   connection_mark_for_close(conn);
   return retval;
 }
@@ -1451,3 +1500,20 @@ connection_dir_finished_connecting(connection_t *conn)
   return 0;
 }
 
+/** Called when one or more networkstatus fetches have failed (with uppercase
+ * fingerprints listed in <b>fp</>).  Mark those fingerprints has having
+ * failed once. */
+static void
+dir_networkstatus_download_failed(smartlist_t *failed)
+{
+  SMARTLIST_FOREACH(failed, const char *, fp,
+  {
+    char digest[DIGEST_LEN];
+    trusted_dir_server_t *dir;
+    base16_decode(digest, DIGEST_LEN, fp, strlen(fp));
+    dir = router_get_trusteddirserver_by_digest(digest);
+
+    ++dir->n_networkstatus_failures;
+  });
+}
+
