@@ -1,8 +1,8 @@
 /* Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: dirserv.c,v 1.220 2005/09/09 21:03:57 nickm Exp $ */
-const char dirserv_c_id[] = "$Id: dirserv.c,v 1.220 2005/09/09 21:03:57 nickm Exp $";
+/* $Id: dirserv.c,v 1.227 2005/09/13 21:14:55 nickm Exp $ */
+const char dirserv_c_id[] = "$Id: dirserv.c,v 1.227 2005/09/13 21:14:55 nickm Exp $";
 
 #include "or.h"
 
@@ -388,11 +388,11 @@ directory_remove_invalid(void)
       changed = 1;
     } else if (r>0 && !ent->is_verified) {
       log(LOG_INFO, "Router '%s' is now approved.", ent->nickname);
-      ent->is_verified = 1;
+      ent->is_verified = ent->is_named = 1;
       changed = 1;
     } else if (r==0 && ent->is_verified) {
       log(LOG_INFO, "Router '%s' is no longer approved.", ent->nickname);
-      ent->is_verified = 0;
+      ent->is_verified = ent->is_named = 0;
       changed = 1;
     }
   }
@@ -554,6 +554,27 @@ list_server_status(smartlist_t *routers, char **router_status_out)
   return 0;
 }
 
+int
+_compare_tor_version_str_ptr(const void **_a, const void **_b)
+{
+  const char *a = *_a, *b = *_b;
+  int ca, cb;
+  tor_version_t va, vb;
+  ca = tor_version_parse(a, &va);
+  cb = tor_version_parse(b, &vb);
+  /* If they both parse, compare them. */
+  if (!ca && !cb)
+    return tor_version_compare(&va,&vb);
+  /* If one parses, it comes first. */
+  if (!ca && cb)
+    return -1;
+  if (ca && !cb)
+    return 1;
+  /* If neither parses, compare strings.  Also, the directory server admin needs
+  ** to be smacked upside the head.  But Tor is tolerant and gentle. */
+  return strcmp(a,b);
+}
+
 /* Given a (possibly empty) list of config_line_t, each line of which contains
  * a list of comma-separated version numbers surrounded by optional space,
  * allocate and return a new string containing the version numbers, in order,
@@ -568,6 +589,7 @@ format_versions_list(config_line_t *ln)
     smartlist_split_string(versions, ln->value, ",",
                            SPLIT_SKIP_SPACE|SPLIT_IGNORE_BLANK, 0);
   }
+  smartlist_sort(versions, _compare_tor_version_str_ptr);
   result = smartlist_join_strings(versions,",",0,NULL);
   SMARTLIST_FOREACH(versions,char *,s,tor_free(s));
   smartlist_free(versions);
@@ -751,7 +773,9 @@ dirserv_set_cached_directory(const char *directory, time_t published,
 
 /** We've just received a v2 network-status for an authoritative directory
  * with fingerprint <b>fp</b> (hex digest, no spaces), published at
- * <b>published</b>.  Store it so we can serve it to others.
+ * <b>published</b>.  Store it so we can serve it to others.  If
+ * <b>directory</b> is NULL, remove the entry with the given fingerprint from
+ * the cache.
  */
 void
 dirserv_set_cached_networkstatus_v2(const char *directory, const char *fp,
@@ -764,14 +788,26 @@ dirserv_set_cached_networkstatus_v2(const char *directory, const char *fp,
   tor_assert(strlen(fp) == HEX_DIGEST_LEN);
 
   if (!(d = strmap_get(cached_v2_networkstatus, fp))) {
+    if (!directory)
+      return;
     d = tor_malloc_zero(sizeof(cached_dir_t));
     strmap_set(cached_v2_networkstatus, fp, d);
   }
 
   tor_assert(d);
-  set_cached_dir(d, tor_strdup(directory), published);
+  if (directory) {
+    set_cached_dir(d, tor_strdup(directory), published);
+  } else {
+    free_cached_dir(d);
+    strmap_remove(cached_v2_networkstatus, fp);
+  }
 }
 
+/** Helper: If we're an authority for the right directory version (the
+ * directory version is determined by <b>is_v1_object</b>), try to regenerate
+ * auth_src as appropriate and return it, falling back to cache_src on
+ * failure.  If we're a cache, return cach_src.
+ */
 static cached_dir_t *
 dirserv_pick_cached_dir_obj(cached_dir_t *cache_src,
                             cached_dir_t *auth_src,
@@ -806,9 +842,10 @@ dirserv_pick_cached_dir_obj(cached_dir_t *cache_src,
  * DIR_REGEN_SLACK_TIME seconds, call <b>regenerate</b>() to make a fresh one.
  * Yields the compressed version of the directory object if <b>compress</b> is
  * set; otherwise return the uncompressed version.  (In either case, sets
- * *<b>out</b> and returns the size of the buffer in *<b>out</b>.
+ * *<b>out</b> and returns the size of the buffer in *<b>out</b>.)
  *
- * DOCDOC is_v1_object
+ * Use <b>is_v1_object</b> to help determine whether we're authoritative for
+ * this kind of object.
  **/
 static size_t
 dirserv_get_obj(const char **out, int compress,
@@ -1259,7 +1296,7 @@ dirserv_get_routerdescs(smartlist_t *descs_out, const char *key)
            smartlist_add(descs_out, router_get_my_routerinfo());
          } else {
            routerinfo_t *ri = router_get_by_digest(d);
-           if (d)
+           if (ri)
              smartlist_add(descs_out,ri);
          }
        });
