@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: main.c,v 1.548 2005/09/08 20:18:14 nickm Exp $ */
-const char main_c_id[] = "$Id: main.c,v 1.548 2005/09/08 20:18:14 nickm Exp $";
+/* $Id: main.c,v 1.549 2005/09/14 02:36:29 nickm Exp $ */
+const char main_c_id[] = "$Id: main.c,v 1.549 2005/09/14 02:36:29 nickm Exp $";
 
 /**
  * \file main.c
@@ -760,7 +760,8 @@ run_scheduled_events(time_t now)
 
   /** 3d. And every 60 seconds, we relaunch listeners if any died. */
   if (!we_are_hibernating() && time_to_check_listeners < now) {
-    retry_all_listeners(0); /* 0 means "only if some died." */
+    /* 0 means "only launch the ones that died." */
+    retry_all_listeners(0, NULL, NULL);
     time_to_check_listeners = now+60;
   }
 
