@@ -2,7 +2,7 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: or.h,v 1.674 2005/09/08 20:36:40 nickm Exp $ */
+/* $Id: or.h,v 1.680 2005/09/14 02:36:29 nickm Exp $ */
 
 /**
  * \file or.h
@@ -11,7 +11,7 @@
 
 #ifndef __OR_H
 #define __OR_H
-#define OR_H_ID "$Id: or.h,v 1.674 2005/09/08 20:36:40 nickm Exp $"
+#define OR_H_ID "$Id: or.h,v 1.680 2005/09/14 02:36:29 nickm Exp $"
 
 #include "orconfig.h"
 #ifdef MS_WINDOWS
@@ -298,11 +298,13 @@ typedef enum {
 #define DIR_CONN_STATE_CLIENT_SENDING 2
 /** State for connection to directory server: reading HTTP response. */
 #define DIR_CONN_STATE_CLIENT_READING 3
+/** State for connection to directory server: happy and finished. */
+#define DIR_CONN_STATE_CLIENT_FINISHED 4
 /** State for connection at directory server: waiting for HTTP request. */
-#define DIR_CONN_STATE_SERVER_COMMAND_WAIT 4
+#define DIR_CONN_STATE_SERVER_COMMAND_WAIT 5
 /** State for connection at directory server: sending HTTP response. */
-#define DIR_CONN_STATE_SERVER_WRITING 5
-#define _DIR_CONN_STATE_MAX 5
+#define DIR_CONN_STATE_SERVER_WRITING 6
+#define _DIR_CONN_STATE_MAX 6
 
 #define _CONTROL_CONN_STATE_MIN 1
 #define CONTROL_CONN_STATE_OPEN_V0 1
@@ -755,17 +757,22 @@ typedef struct {
   addr_policy_t *exit_policy; /**< What streams will this OR permit
                                       * to exit? */
   long uptime; /**< How many seconds the router claims to have been up */
-  uint8_t is_hibernating; /**< Whether the router claims to be hibernating */
   smartlist_t *declared_family; /**< Nicknames of router which this router
                                  * claims are its family. */
   char *contact_info; /**< Declared contact info for this router. */
+  unsigned int is_hibernating:1; /**< Whether the router claims to be
+                                  * hibernating */
 
   /* local info */
-  int is_running; /**< As far as we know, is this OR currently running? */
+  unsigned int is_running:1; /**< As far as we know, is this OR currently
+                              * running? */
+  unsigned int is_verified:1; /**< Has a trusted dirserver validated this OR?
+                               *  (For Authdir: Have we validated this OR?)
+                               */
+  /*XXXX Make this get used once we think we do naming right. NM */
+  unsigned int is_named:1; /* Do we believe the nickname that this OR gives us? */
+
   time_t status_set_at; /**< When did we last update is_running? */
-  int is_verified; /**< Has a trusted dirserver validated this OR?
-                    * (For Authdir: Have we validated this OR?)
-                    */
 
   /* The below items are used only by authdirservers for
    * reachability testing. */
@@ -828,7 +835,8 @@ typedef struct networkstatus_t {
 
   int binds_names:1; /**< True iff this directory server binds names. */
 
-  smartlist_t *entries; /**< List of router_status_t*. */
+  smartlist_t *entries; /**< List of router_status_t*.   This list is kept
+                         * sorted by identity_digest. */
 } networkstatus_t;
 
 /** Contents of a directory of onion routers. */
@@ -1431,13 +1439,13 @@ extern unsigned long stats_n_destroy_cells_processed;
 /********************************* config.c ***************************/
 
 or_options_t *get_options(void);
-void set_options(or_options_t *new_val);
+int set_options(or_options_t *new_val);
 void config_free_all(void);
 const char *safe_str(const char *address);
 
 int config_get_lines(char *string, config_line_t **result);
 void config_free_lines(config_line_t *front);
-int options_trial_assign(config_line_t *list, int reset);
+int options_trial_assign(config_line_t *list, int use_defaults, int clear_first);
 int resolve_my_address(or_options_t *options, uint32_t *addr,
                        char **hostname_out);
 void options_init(or_options_t *options);
@@ -1484,7 +1492,8 @@ void _connection_mark_for_close(connection_t *conn,int line, const char *file);
 void connection_expire_held_open(void);
 
 int connection_connect(connection_t *conn, char *address, uint32_t addr, uint16_t port);
-int retry_all_listeners(int force);
+int retry_all_listeners(int force, smartlist_t *replaced_conns,
+                        smartlist_t *new_conns);
 
 void connection_bucket_init(void);
 void connection_bucket_refill(struct timeval *now);
@@ -1504,6 +1513,9 @@ connection_t *connection_get_by_identity_digest(const char *digest, int type);
 connection_t *connection_get_by_global_id(uint32_t id);
 
 connection_t *connection_get_by_type(int type);
+connection_t *connection_get_by_type_purpose(int type, int purpose);
+connection_t *connection_get_by_type_addr_port_purpose(int type, uint32_t addr,
+                                                     uint16_t port, int purpose);
 connection_t *connection_get_by_type_state(int type, int state);
 connection_t *connection_get_by_type_state_lastwritten(int type, int state);
 connection_t *connection_get_by_type_state_rendquery(int type, int state, const char *rendquery);
@@ -1689,7 +1701,7 @@ int connection_dir_reached_eof(connection_t *conn);
 int connection_dir_process_inbuf(connection_t *conn);
 int connection_dir_finished_flushing(connection_t *conn);
 int connection_dir_finished_connecting(connection_t *conn);
-void connection_dir_connect_failed(connection_t *conn);
+void connection_dir_request_failed(connection_t *conn);
 void parse_dir_policy(void);
 void free_dir_policy(void);
 
@@ -2010,13 +2022,18 @@ void router_free_all_keys(void);
 
 /********************************* routerlist.c ***************************/
 
+/** Represents information about a single trusted directory server. */
 typedef struct trusted_dir_server_t {
-  char *address;
-  uint32_t addr;
-  uint16_t dir_port;
-  char digest[DIGEST_LEN];
-  int is_running;
-  int supports_v1_protocol;
+  char *address; /**< Hostname */
+  uint32_t addr; /**< IPv4 address */
+  uint16_t dir_port; /**< Directory port */
+  char digest[DIGEST_LEN]; /**< Digest of identity key */
+  unsigned int is_running:1; /**< True iff we think this server is running. */
+  unsigned int supports_v1_protocol:1; /**< True iff this server is an
+                                        * authority for the older ("v1")
+                                        * directory protocol.*/
+  int n_networkstatus_failures; /**< How many times have we asked for this
+                                 * server's network-status unsuccessfully? */
 } trusted_dir_server_t;
 
 int router_reload_router_list(void);
@@ -2097,6 +2114,8 @@ void clear_trusted_dir_servers(void);
 networkstatus_t *networkstatus_get_by_digest(const char *digest);
 void update_networkstatus_cache_downloads(time_t now);
 void update_networkstatus_client_downloads(time_t now);
+void routers_update_status_from_networkstatus(smartlist_t *routers);
+smartlist_t *router_list_superseded(void);
 
 /********************************* routerparse.c ************************/
 
