@@ -1,7 +1,7 @@
 /* Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: rephist.c,v 1.62 2005/07/25 10:29:21 arma Exp $ */
-const char rephist_c_id[] = "$Id: rephist.c,v 1.62 2005/07/25 10:29:21 arma Exp $";
+/* $Id: rephist.c,v 1.63 2005/08/13 00:22:07 arma Exp $ */
+const char rephist_c_id[] = "$Id: rephist.c,v 1.63 2005/08/13 00:22:07 arma Exp $";
 
 /**
  * \file rephist.c
@@ -748,8 +748,11 @@ rep_hist_note_used_hidserv(time_t now, int need_uptime, int need_capacity)
 int
 rep_hist_get_predicted_hidserv(time_t now, int *need_uptime, int *need_capacity)
 {
-  if (!predicted_hidserv_time) /* initialize it */
+  if (!predicted_hidserv_time) { /* initialize it */
     predicted_hidserv_time = now;
+    predicted_hidserv_uptime_time = now;
+    predicted_hidserv_capacity_time = now;
+  }
   if (predicted_hidserv_time + PREDICTED_CIRCS_RELEVANCE_TIME < now)
     return 0; /* too long ago */
   if (predicted_hidserv_uptime_time + PREDICTED_CIRCS_RELEVANCE_TIME < now)
