@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: router.c,v 1.205 2005/09/08 20:18:15 nickm Exp $ */
-const char router_c_id[] = "$Id: router.c,v 1.205 2005/09/08 20:18:15 nickm Exp $";
+/* $Id: router.c,v 1.207 2005/09/13 21:39:42 arma Exp $ */
+const char router_c_id[] = "$Id: router.c,v 1.207 2005/09/13 21:39:42 arma Exp $";
 
 #include "or.h"
 
@@ -433,7 +433,7 @@ router_orport_found_reachable(void)
 {
   if (!can_reach_or_port) {
     if (!clique_mode(get_options()))
-      log(LOG_NOTICE,"Your ORPort is reachable from the outside. Excellent.%s",
+      log(LOG_NOTICE,"Self-testing indicates your ORPort is reachable from the outside. Excellent.%s",
           get_options()->NoPublish ? "" : " Publishing server descriptor.");
     can_reach_or_port = 1;
     mark_my_descriptor_dirty();
@@ -446,7 +446,7 @@ void
 router_dirport_found_reachable(void)
 {
   if (!can_reach_dir_port) {
-    log(LOG_NOTICE,"Your DirPort is reachable from the outside. Excellent.");
+    log(LOG_NOTICE,"Self-testing indicates your DirPort is reachable from the outside. Excellent.");
     can_reach_dir_port = 1;
   }
 }
@@ -764,10 +764,13 @@ router_rebuild_descriptor(int force)
   config_parse_addr_policy(get_options()->ExitPolicy, &ri->exit_policy, -1);
   options_append_default_exit_policy(&ri->exit_policy);
 
-  if (desc_routerinfo) /* inherit values */
+  if (desc_routerinfo) { /* inherit values */
     ri->is_verified = desc_routerinfo->is_verified;
+    ri->is_running = desc_routerinfo->is_running;
+    ri->is_named = desc_routerinfo->is_named;
+  }
   if (authdir_mode(options))
-    ri->is_verified = 1; /* believe in yourself */
+    ri->is_verified = ri->is_named = 1; /* believe in yourself */
   if (options->MyFamily) {
     ri->declared_family = smartlist_create();
     smartlist_split_string(ri->declared_family, options->MyFamily, ",",
