@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: routerlist.c,v 1.280 2005/09/08 21:39:39 nickm Exp $ */
-const char routerlist_c_id[] = "$Id: routerlist.c,v 1.280 2005/09/08 21:39:39 nickm Exp $";
+/* $Id: routerlist.c,v 1.286 2005/09/13 21:18:00 nickm Exp $ */
+const char routerlist_c_id[] = "$Id: routerlist.c,v 1.286 2005/09/13 21:18:00 nickm Exp $";
 
 /**
  * \file routerlist.c
@@ -16,15 +16,16 @@ const char routerlist_c_id[] = "$Id: routerlist.c,v 1.280 2005/09/08 21:39:39 ni
 
 /****************************************************************************/
 
+/** Global list of a trusted_dir_server_t object for each trusted directory
+ * server. */
 static smartlist_t *trusted_dir_servers = NULL;
 
 /* static function prototypes */
-static routerinfo_t *
-router_pick_directory_server_impl(int requireother, int fascistfirewall,
-                                  int for_v2_directory);
-static trusted_dir_server_t *
-router_pick_trusteddirserver_impl(int need_v1_support,
-                                  int requireother, int fascistfirewall);
+static routerinfo_t *router_pick_directory_server_impl(int requireother,
+                                                       int fascistfirewall,
+                                                       int for_v2_directory);
+static trusted_dir_server_t *router_pick_trusteddirserver_impl(
+                int need_v1_support, int requireother, int fascistfirewall);
 static void mark_all_trusteddirservers_up(void);
 static int router_nickname_is_in_list(routerinfo_t *router, const char *list);
 static int router_nickname_matches(routerinfo_t *router, const char *nickname);
@@ -44,7 +45,7 @@ static routerlist_t *routerlist = NULL;
 extern int has_fetched_directory; /**< from main.c */
 
 /** Global list of all of the current network_status documents that we know
- * about. */
+ * about.  This list is kept sorted by published_on. */
 static smartlist_t *networkstatus_list = NULL;
 
 /**
@@ -79,7 +80,8 @@ router_reload_router_list(void)
   return 0;
 }
 
-/** DOCDOC */
+/** Repopulate our list of network_status_t objects from the list cached on
+ * disk.  Return 0 on success, -1 on failure. */
 int
 router_reload_networkstatus(void)
 {
@@ -117,7 +119,122 @@ router_reload_networkstatus(void)
   return 0;
 }
 
-/* Set *<b>outp</b> to a smartlist containing a list of
+/* Router descriptor storage.
+ *
+ * Routerdescs are stored in a big file, named "cached-routers".  As new
+ * routerdescs arrive, we append them to a journal file named
+ * "cached-routers.jrn".
+ *
+ * From time to time, we replace "cached-routers" with a new file containing
+ * only the live, non-superseded descriptors, and clear cached-routers.log.
+ *
+ * On startup, we read both files.
+ */
+
+/** The size of the router log, in bytes. */
+static size_t router_journal_len = 0;
+/** The size of the router store, in bytes. */
+static size_t router_store_len = 0;
+
+/** Helper: return 1 iff the router log is so big we want to rebuild the
+ * store. */
+static int
+router_should_rebuild_store(void)
+{
+  if (router_store_len > (1<<16))
+    return router_journal_len > router_store_len / 2;
+  else
+    return router_journal_len > (1<<15);
+}
+
+/** Add the <b>len</b>-type router descriptor in <b>s</b> to the router
+ * journal. */
+int
+router_append_to_journal(const char *s, size_t len)
+{
+  or_options_t *options = get_options();
+  size_t fname_len = strlen(options->DataDirectory)+32;
+  char *fname = tor_malloc(len);
+
+  tor_snprintf(fname, fname_len, "%s/cached-routers.new",
+               options->DataDirectory);
+
+  if (!len)
+    len = strlen(s);
+
+  if (append_bytes_to_file(fname, s, len, 0)) {
+    log_fn(LOG_WARN, "Unable to store router descriptor");
+    tor_free(fname);
+    return -1;
+  }
+
+  tor_free(fname);
+  router_journal_len += len;
+  return 0;
+}
+
+/** If the journal is too long, or if <b>force</b> is true, then atomically
+ * replace the router store with the routers currently in our routerlist, and
+ * clear the journal.  Return 0 on success, -1 on failure.
+ */
+int
+router_rebuild_store(int force)
+{
+  size_t len = 0;
+  or_options_t *options;
+  size_t fname_len;
+  smartlist_t *chunk_list = NULL;
+  char *fname = NULL;
+  int r = -1;
+
+  if (!force && !router_should_rebuild_store())
+    return 0;
+  if (!routerlist)
+    return 0;
+
+  options = get_options();
+  fname_len = strlen(options->DataDirectory)+32;
+  fname = tor_malloc(fname_len);
+  tor_snprintf(fname, fname_len, "%s/cached-routers", options->DataDirectory);
+  chunk_list = smartlist_create();
+
+  SMARTLIST_FOREACH(routerlist->routers, routerinfo_t *, ri,
+  {
+    sized_chunk_t *c;
+    if (!ri->signed_descriptor) {
+      log_fn(LOG_WARN, "Bug! No descriptor stored for router '%s'.",
+             ri->nickname);
+      goto done;
+    }
+    c = tor_malloc(sizeof(sized_chunk_t));
+    c->bytes = ri->signed_descriptor;
+    c->len = ri->signed_descriptor_len;
+    smartlist_add(chunk_list, c);
+  });
+
+  if (write_chunks_to_file(fname, chunk_list, 0)<0) {
+    log_fn(LOG_WARN, "Error writing router store to disk.");
+    goto done;
+  }
+
+  tor_snprintf(fname, fname_len, "%s/cached-routers.new",
+               options->DataDirectory);
+
+  write_str_to_file(fname, "", 0);
+
+  r = 0;
+  router_store_len = len;
+  router_journal_len = 0;
+ done:
+  tor_free(fname);
+  if (chunk_list) {
+    SMARTLIST_FOREACH(chunk_list, sized_chunk_t *, c, tor_free(c));
+    smartlist_free(chunk_list);
+  }
+  return r;
+}
+
+/** Set *<b>outp</b> to a smartlist containing a list of
  * trusted_dir_server_t * for all known trusted dirservers.  Callers
  * must not modify the list or its contents.
  */
@@ -195,8 +312,6 @@ router_get_trusteddirserver_by_digest(const char *digest)
  * trusted dirservers and <b>retry_if_no_servers</b> is non-zero,
  * set them all as running again, and try again.
  * Other args are as in router_pick_trusteddirserver_impl().
- *
- * DOCDOC need_v1_support
  */
 trusted_dir_server_t *
 router_pick_trusteddirserver(int need_v1_support,
@@ -221,16 +336,15 @@ router_pick_trusteddirserver(int need_v1_support,
  * routerlist.
  * If <b>fascistfirewall</b> and we're not using a proxy,
  * make sure the port we pick is allowed by options-\>firewallports.
- * If <b>requireother</b>, it cannot be us.
- *
- * DOCDOC need_v1_support, for_v2_directory
+ * If <b>requireother</b>, it cannot be us.  If <b>for_v2_directory</b>,
+ * choose a directory server new enough to support the v2 directory
+ * functionality.
  */
 static routerinfo_t *
 router_pick_directory_server_impl(int requireother, int fascistfirewall,
                                   int for_v2_directory)
 {
-  int i;
-  routerinfo_t *router;
+  routerinfo_t *result;
   smartlist_t *sl;
 
   if (!routerlist)
@@ -241,8 +355,8 @@ router_pick_directory_server_impl(int requireother, int fascistfirewall,
 
   /* Find all the running dirservers we know about. */
   sl = smartlist_create();
-  for (i=0;i < smartlist_len(routerlist->routers); i++) {
-    router = smartlist_get(routerlist->routers, i);
+  SMARTLIST_FOREACH(routerlist->routers, routerinfo_t *, router,
+  {
     if (!router->is_running || !router->dir_port || !router->is_verified)
       continue;
     if (requireother && router_is_me(router))
@@ -257,17 +371,18 @@ router_pick_directory_server_impl(int requireother, int fascistfirewall,
           router_digest_is_trusted_dir(router->identity_digest)))
       continue;
     smartlist_add(sl, router);
-  }
+  });
 
-  router = smartlist_choose(sl);
+  result = smartlist_choose(sl);
   smartlist_free(sl);
-  return router;
+  return result;
 }
 
 /** Choose randomly from among the trusted dirservers that are up.
  * If <b>fascistfirewall</b> and we're not using a proxy,
  * make sure the port we pick is allowed by options-\>firewallports.
- * If <b>requireother</b>, it cannot be us.
+ * If <b>requireother</b>, it cannot be us.  If <b>need_v1_support</b>, choose
+ * a trusted authority for the v1 directory system.
  */
 static trusted_dir_server_t *
 router_pick_trusteddirserver_impl(int need_v1_support,
@@ -458,14 +573,11 @@ static void
 router_add_running_routers_to_smartlist(smartlist_t *sl, int allow_unverified,
                                         int need_uptime, int need_capacity)
 {
-  routerinfo_t *router;
-  int i;
-
   if (!routerlist)
     return;
 
-  for (i=0;i<smartlist_len(routerlist->routers);i++) {
-    router = smartlist_get(routerlist->routers, i);
+  SMARTLIST_FOREACH(routerlist->routers, routerinfo_t *, router,
+  {
     if (router->is_running &&
         (router->is_verified ||
         (allow_unverified &&
@@ -475,7 +587,7 @@ router_add_running_routers_to_smartlist(smartlist_t *sl, int allow_unverified,
        */
       smartlist_add(sl, router);
     }
-  }
+  });
 }
 
 /** Look through the routerlist until we find a router that has my key.
@@ -483,17 +595,14 @@ router_add_running_routers_to_smartlist(smartlist_t *sl, int allow_unverified,
 routerinfo_t *
 routerlist_find_my_routerinfo(void)
 {
-  routerinfo_t *router;
-  int i;
-
   if (!routerlist)
     return NULL;
 
-  for (i=0;i<smartlist_len(routerlist->routers);i++) {
-    router = smartlist_get(routerlist->routers, i);
+  SMARTLIST_FOREACH(routerlist->routers, routerinfo_t *, router,
+  {
     if (router_is_me(router))
       return router;
-  }
+  });
   return NULL;
 }
 
@@ -502,9 +611,8 @@ routerlist_find_my_routerinfo(void)
  * isn't a good one.
  */
 routerinfo_t *
-router_find_exact_exit_enclave(const char *address, uint16_t port) {
-  int i;
-  routerinfo_t *router;
+router_find_exact_exit_enclave(const char *address, uint16_t port)
+{
   uint32_t addr;
   struct in_addr in;
 
@@ -512,8 +620,8 @@ router_find_exact_exit_enclave(const char *address, uint16_t port) {
     return NULL; /* it's not an IP already */
   addr = ntohl(in.s_addr);
 
-  for (i=0;i < smartlist_len(routerlist->routers); i++) {
-    router = smartlist_get(routerlist->routers, i);
+  SMARTLIST_FOREACH(routerlist->routers, routerinfo_t *, router,
+  {
     log_fn(LOG_DEBUG,"Considering %s: %d, %u==%u, %d.",
            router->nickname,
            router->is_running,
@@ -524,7 +632,7 @@ router_find_exact_exit_enclave(const char *address, uint16_t port) {
         router_compare_addr_to_addr_policy(addr, port, router->exit_policy) ==
           ADDR_POLICY_ACCEPTED)
       return router;
-  }
+  });
   return NULL;
 }
 
@@ -703,8 +811,7 @@ router_nickname_matches(routerinfo_t *router, const char *nickname)
 routerinfo_t *
 router_get_by_nickname(const char *nickname)
 {
-  int i, maybedigest;
-  routerinfo_t *router;
+  int maybedigest;
   char digest[DIGEST_LEN];
 
   tor_assert(nickname);
@@ -719,13 +826,13 @@ router_get_by_nickname(const char *nickname)
   maybedigest = (strlen(nickname) == HEX_DIGEST_LEN) &&
     (base16_decode(digest,DIGEST_LEN,nickname,HEX_DIGEST_LEN) == 0);
 
-  for (i=0;i<smartlist_len(routerlist->routers);i++) {
-    router = smartlist_get(routerlist->routers, i);
+  SMARTLIST_FOREACH(routerlist->routers, routerinfo_t *, router,
+  {
     if (0 == strcasecmp(router->nickname, nickname) ||
         (maybedigest && 0 == memcmp(digest, router->identity_digest,
                                     DIGEST_LEN)))
       return router;
-  }
+  });
 
   return NULL;
 }
@@ -766,18 +873,15 @@ router_get_by_hexdigest(const char *hexdigest)
 routerinfo_t *
 router_get_by_digest(const char *digest)
 {
-  int i;
-  routerinfo_t *router;
-
   tor_assert(digest);
 
   if (!routerlist) return NULL;
 
-  for (i=0;i<smartlist_len(routerlist->routers);i++) {
-    router = smartlist_get(routerlist->routers, i);
+  SMARTLIST_FOREACH(routerlist->routers, routerinfo_t*, router,
+  {
     if (0 == memcmp(router->identity_digest, digest, DIGEST_LEN))
       return router;
-  }
+  });
 
   return NULL;
 }
@@ -980,6 +1084,7 @@ router_add_to_routerlist(routerinfo_t *router, const char **msg)
       return -2;
     }
     router->is_verified = authdir_verified;
+    router->is_named = router->is_verified; /*XXXX NM not right. */
     if (tor_version_as_new_as(router->platform,"0.1.0.2-rc"))
       router->is_verified = 1;
   }
@@ -996,6 +1101,7 @@ router_add_to_routerlist(routerinfo_t *router, const char **msg)
         if (authdir) {
           /* Update the is_verified status based on our lookup. */
           old_router->is_verified = router->is_verified;
+          old_router->is_named = router->is_named;
         } else {
           /* Update the is_running status to whatever we were told. */
           old_router->is_running = router->is_running;
@@ -1020,11 +1126,11 @@ router_add_to_routerlist(routerinfo_t *router, const char **msg)
             dirserv_thinks_router_is_blatantly_unreachable(router, time(NULL))) {
           if (router->num_unreachable_notifications >= 3) {
             unreachable = 1;
-            log_fn(LOG_WARN, "Notifying server '%s' that it's unreachable. (ContactInfo '%s', platform '%s').",
+            log_fn(LOG_NOTICE, "Notifying server '%s' that it's unreachable. (ContactInfo '%s', platform '%s').",
               router->nickname, router->contact_info ? router->contact_info : "",
               router->platform ? router->platform : "");
           } else {
-            log_fn(LOG_NOTICE,"'%s' may be unreachable -- the %d previous descriptors were thought to be unreachable.", router->nickname, router->num_unreachable_notifications);
+            log_fn(LOG_INFO,"'%s' may be unreachable -- the %d previous descriptors were thought to be unreachable.", router->nickname, router->num_unreachable_notifications);
             router->num_unreachable_notifications++;
           }
         }
@@ -1198,10 +1304,54 @@ router_load_routerlist_from_directory(const char *s,
   return 0;
 }
 
-/** How far in the future do we allow a network-status to get? (seconds) */
+/** Helper: return a newly allocated string containing the name of the filename
+ * where we plan to cache <b>ns</b>. */
+static char *
+networkstatus_get_cache_filename(const networkstatus_t *ns)
+{
+  const char *datadir = get_options()->DataDirectory;
+  size_t len = strlen(datadir)+64;
+  char fp[HEX_DIGEST_LEN+1];
+  char *fn = tor_malloc(len+1);
+  base16_encode(fp, HEX_DIGEST_LEN+1, ns->identity_digest, DIGEST_LEN);
+  tor_snprintf(fn, len, "%s/cached-status/%s",datadir,fp);
+  return fn;
+
+}
+
+/** Helper for smartlist_sort: Compare two networkstatus objects by
+ * publication date. */
+static int
+_compare_networkstatus_published_on(const void **_a, const void **_b)
+{
+  const networkstatus_t *a = *_a, *b = *_b;
+  if (a->published_on < b->published_on)
+    return -1;
+  else if (a->published_on > b->published_on)
+    return 1;
+  else
+    return 0;
+}
+
+/** How far in the future do we allow a network-status to get before removing
+ * it? (seconds) */
 #define NETWORKSTATUS_ALLOW_SKEW (48*60*60)
-/** DOCDOC returns 0 on no problems, -1 on problems.
- * requested fingerprints must be upcased.
+/** Given a string <b>s</b> containing a network status that we received at
+ * <b>arrived_at</b> from <b>source</b>, try to parse it, see if we want to
+ * store it, and put it into our cache is necessary.
+ *
+ * If <b>source</b> is NS_FROM_DIR or NS_FROM_CACHE, do not replace our
+ * own networkstatus_t (if we're a directory server).
+ *
+ * If <b>source</b> is NS_FROM_CACHE, do not write our networkstatus_t to the
+ * cache.
+ *
+ * If <b>requested_fingerprints</b> is provided, it must contain a list of
+ * uppercased identity fingerprints.  Do not update any networkstatus whose
+ * fingerprint is not on the list; after updating a networkstatus, remove its
+ * fingerprint from the list.
+ *
+ * Return 0 on success, -1 on failure.
  */
 int
 router_set_networkstatus(const char *s, time_t arrived_at,
@@ -1210,15 +1360,16 @@ router_set_networkstatus(const char *s, time_t arrived_at,
   networkstatus_t *ns;
   int i, found;
   time_t now;
-  char fp[HEX_DIGEST_LEN+1];
   int skewed = 0;
+  trusted_dir_server_t *trusted_dir;
+  char fp[HEX_DIGEST_LEN+1];
 
   ns = networkstatus_parse_from_string(s);
   if (!ns) {
     log_fn(LOG_WARN, "Couldn't parse network status.");
     return -1;
   }
-  if (!router_digest_is_trusted_dir(ns->identity_digest)) {
+  if (!(trusted_dir=router_get_trusteddirserver_by_digest(ns->identity_digest))) {
     log_fn(LOG_INFO, "Network status was signed, but not by an authoritative directory we recognize.");
     networkstatus_free(ns);
     return -1;
@@ -1238,21 +1389,27 @@ router_set_networkstatus(const char *s, time_t arrived_at,
     networkstatus_list = smartlist_create();
 
   if (source == NS_FROM_DIR && router_digest_is_me(ns->identity_digest)) {
-    /* Drop our own networkstatus when we get it from somebody else. */
+    /* Don't replace our own networkstatus when we get it from somebody else. */
     networkstatus_free(ns);
     return 0;
   }
 
   base16_encode(fp, HEX_DIGEST_LEN+1, ns->identity_digest, DIGEST_LEN);
 
-  if (requested_fingerprints &&
-      !smartlist_string_isin(requested_fingerprints, fp)) {
-    char *requested = smartlist_join_strings(requested_fingerprints," ",0,NULL);
-    log_fn(LOG_WARN, "We received a network status with a fingerprint (%s) that we never requested. (%s) Dropping.", fp, requested);
-    tor_free(requested);
-    return 0;
+  if (requested_fingerprints) {
+    if (smartlist_string_isin(requested_fingerprints, fp)) {
+      smartlist_string_remove(requested_fingerprints, fp);
+    } else {
+      char *requested = smartlist_join_strings(requested_fingerprints," ",0,NULL);
+      log_fn(LOG_WARN, "We received a network status with a fingerprint (%s) that we never requested. (%s) Dropping.", fp, requested);
+      tor_free(requested);
+      return 0;
+    }
   }
 
+  if (source != NS_FROM_CACHE)
+    trusted_dir->n_networkstatus_failures = 0;
+
   found = 0;
   for (i=0; i < smartlist_len(networkstatus_list); ++i) {
     networkstatus_t *old_ns = smartlist_get(networkstatus_list, i);
@@ -1281,11 +1438,10 @@ router_set_networkstatus(const char *s, time_t arrived_at,
   if (!found)
     smartlist_add(networkstatus_list, ns);
 
+  smartlist_sort(networkstatus_list, _compare_networkstatus_published_on);
+
   if (source != NS_FROM_CACHE && !skewed) {
-    const char *datadir = get_options()->DataDirectory;
-    size_t len = strlen(datadir)+64;
-    char *fn = tor_malloc(len+1);
-    tor_snprintf(fn, len, "%s/cached-status/%s",datadir,fp);
+    char *fn = networkstatus_get_cache_filename(ns);
     if (write_str_to_file(fn, s, 0)<0) {
       log_fn(LOG_NOTICE, "Couldn't write cached network status to \"%s\"", fn);
     }
@@ -1298,10 +1454,66 @@ router_set_networkstatus(const char *s, time_t arrived_at,
   return 0;
 }
 
+/** How old do we allow a network-status to get before removing it completely? */
+#define MAX_NETWORKSTATUS_AGE (10*24*60*60)
+/** Remove all very-old network_status_t objects from memory and from the
+ * disk cache. */
+void
+networkstatus_list_clean(time_t now)
+{
+  int i;
+  if (!networkstatus_list)
+    return;
+
+  for (i = 0; i < smartlist_len(networkstatus_list); ++i) {
+    networkstatus_t *ns = smartlist_get(networkstatus_list, i);
+    char *fname = NULL;;
+    if (ns->published_on + MAX_NETWORKSTATUS_AGE > now)
+      continue;
+    /* Okay, this one is too old.  Remove it from the list, and delete it
+     * from the cache. */
+    smartlist_del(networkstatus_list, i--);
+    fname = networkstatus_get_cache_filename(ns);
+    if (file_status(fname) == FN_FILE) {
+      log_fn(LOG_INFO, "Removing too-old networkstatus in %s", fname);
+      unlink(fname);
+    }
+    tor_free(fname);
+    if (get_options()->DirPort) {
+      char fp[HEX_DIGEST_LEN+1];
+      base16_encode(fp, sizeof(fp), ns->identity_digest, DIGEST_LEN);
+      dirserv_set_cached_networkstatus_v2(NULL, fp, 0);
+    }
+    networkstatus_free(ns);
+  }
+}
+
+/** Helper for bsearching a list of routerstatus_t pointers.*/
+static int
+_compare_digest_to_routerstatus_entry(const void *_key, const void **_member)
+{
+  const char *key = _key;
+  const routerstatus_t *rs = *_member;
+  return memcmp(key, rs->identity_digest, DIGEST_LEN);
+}
+
+/** Return the entry in <b>ns</b> for the identity digest <b>digest</b>, or
+ * NULL if none was found. */
+routerstatus_t *
+networkstatus_find_entry(networkstatus_t *ns, const char *digest)
+{
+  return smartlist_bsearch(ns->entries, digest,
+                           _compare_digest_to_routerstatus_entry);
+}
+
 /* XXXX These should be configurable, perhaps? NM */
 #define AUTHORITY_NS_CACHE_INTERVAL 10*60
 #define NONAUTHORITY_NS_CACHE_INTERVAL 15*60
-/* DOCDOC*/
+/** We are a directory server, and so cache network_status documents.
+ * Initiate downloads as needed to update them.  For authorities, this means
+ * asking each trusted directory for its network-status.  For caches, this means
+ * asking a random authority for all network-statuses.
+ */
 void
 update_networkstatus_cache_downloads(time_t now)
 {
@@ -1310,6 +1522,7 @@ update_networkstatus_cache_downloads(time_t now)
   int interval =
     authority ? AUTHORITY_NS_CACHE_INTERVAL : NONAUTHORITY_NS_CACHE_INTERVAL;
 
+  /*XXXXX NM we should retry on failure. */
   if (last_downloaded + interval >= now)
     return;
   if (!trusted_dir_servers)
@@ -1324,6 +1537,12 @@ update_networkstatus_cache_downloads(time_t now)
          char resource[HEX_DIGEST_LEN+6];
          if (router_digest_is_me(ds->digest))
            continue;
+         if (connection_get_by_type_addr_port_purpose(
+                CONN_TYPE_DIR, ds->addr, ds->dir_port,
+                DIR_PURPOSE_FETCH_NETWORKSTATUS)) {
+           /* We are already fetching this one. */
+           continue;
+         }
          strlcpy(resource, "fp/", sizeof(resource));
          base16_encode(resource+3, sizeof(resource)-3, ds->digest, DIGEST_LEN);
          strlcat(resource, ".z", sizeof(resource));
@@ -1331,7 +1550,10 @@ update_networkstatus_cache_downloads(time_t now)
        });
   } else {
     /* A non-authority cache launches one connection to a random authority. */
-    directory_get_from_dirserver(DIR_PURPOSE_FETCH_NETWORKSTATUS,"all.z",1);
+    /* (Check whether we're currently fetching network-status objects.) */
+    if (!connection_get_by_type_purpose(CONN_TYPE_DIR,
+                                        DIR_PURPOSE_FETCH_NETWORKSTATUS))
+      directory_get_from_dirserver(DIR_PURPOSE_FETCH_NETWORKSTATUS,"all.z",1);
   }
 }
 
@@ -1342,7 +1564,10 @@ update_networkstatus_cache_downloads(time_t now)
  * before downloading the next in sequence? */
 #define NETWORKSTATUS_CLIENT_DL_INTERVAL (30*60)
 
-/** DOCDOC */
+/** We are not a directory cache or authority.  Update our network-status list
+ * by launching a new directory fetch for enough network-status documents "as
+ * necessary".  See function comments for implementation details.
+ */
 void
 update_networkstatus_client_downloads(time_t now)
 {
@@ -1353,10 +1578,15 @@ update_networkstatus_client_downloads(time_t now)
   char *resource, *cp;
   size_t resource_len;
 
+  if (connection_get_by_type_purpose(CONN_TYPE_DIR,
+                                     DIR_PURPOSE_FETCH_NETWORKSTATUS))
+    return;
+
   /* This is a little tricky.  We want to download enough network-status
-   * objects so that we have at least half of them under NETWORKSTATUS_MAX_VALIDITY
-   * publication time.  We want to download a new *one* if the most recent
-   * one's publication time is under NETWORKSTATUS_CLIENT_DL_INTERVAL.
+   * objects so that we have at least half of them under
+   * NETWORKSTATUS_MAX_VALIDITY publication time.  We want to download a new
+   * *one* if the most recent one's publication time is under
+   * NETWORKSTATUS_CLIENT_DL_INTERVAL.
    */
   if (!trusted_dir_servers || !smartlist_len(trusted_dir_servers))
     return;
@@ -1369,7 +1599,7 @@ update_networkstatus_client_downloads(time_t now)
        if (ns->published_on > now-NETWORKSTATUS_MAX_VALIDITY)
          ++n_live;
        if (!most_recent || ns->received_on > most_recent_received) {
-         most_recent_idx = ds_sl_idx; /* magic variable from FOREACH*/
+         most_recent_idx = ds_sl_idx; /* magic variable from FOREACH */
          most_recent = ds;
          most_recent_received = ns->received_on;
        }
@@ -1517,20 +1747,18 @@ int
 router_exit_policy_all_routers_reject(uint32_t addr, uint16_t port,
                                           int need_uptime)
 {
-  int i;
-  routerinfo_t *router;
   addr_policy_result_t r;
   if (!routerlist) return 1;
 
-  for (i=0;i<smartlist_len(routerlist->routers);i++) {
-    router = smartlist_get(routerlist->routers, i);
+  SMARTLIST_FOREACH(routerlist->routers, routerinfo_t *, router,
+  {
     if (router->is_running &&
         !router_is_unreliable(router, need_uptime, 0)) {
       r = router_compare_addr_to_addr_policy(addr, port, router->exit_policy);
       if (r != ADDR_POLICY_REJECTED && r != ADDR_POLICY_PROBABLY_REJECTED)
         return 0; /* this one could be ok. good enough. */
     }
-  }
+  });
   return 1; /* all will reject. */
 }
 
@@ -1707,7 +1935,7 @@ routers_update_status_from_entry(smartlist_t *routers,
 {
   int authdir = get_options()->AuthoritativeDir;
   int is_running = 1;
-  int is_verified = 0;
+  int is_verified = 0, is_named = 0;
   int hex_digest_set = 0;
   char nickname[MAX_NICKNAME_LEN+1];
   char hexdigest[HEX_DIGEST_LEN+1];
@@ -1726,6 +1954,7 @@ routers_update_status_from_entry(smartlist_t *routers,
      * entry will either extend to a NUL (old running-routers format) or to an
      * equals sign (new router-status format). */
     is_verified = 1;
+    is_named = 1;
     end = strchr(cp, '=');
     if (!end)
       end = strchr(cp,'\0');
@@ -1781,9 +2010,9 @@ routers_update_status_from_entry(smartlist_t *routers,
       /* If we're not an authoritative directory, update verified status.
        */
       if (nickname_matches && digest_matches)
-        r->is_verified = 1;
+        r->is_verified = r->is_named = 1;
       else if (digest_matches)
-        r->is_verified = 0;
+        r->is_verified = r->is_named = 0;
     }
     if (digest_matches)
       if (r->status_set_at < list_time) {
@@ -1844,7 +2073,7 @@ add_trusted_dir_server(const char *address, uint16_t port, const char *digest,
     a = ntohl(a);
   }
 
-  ent = tor_malloc(sizeof(trusted_dir_server_t));
+  ent = tor_malloc_zero(sizeof(trusted_dir_server_t));
   ent->address = hostname;
   ent->addr = a;
   ent->dir_port = port;
@@ -1867,6 +2096,7 @@ clear_trusted_dir_servers(void)
   }
 }
 
+/** Return the network status with a given identity digest. */
 networkstatus_t *
 networkstatus_get_by_digest(const char *digest)
 {
@@ -1878,3 +2108,141 @@ networkstatus_get_by_digest(const char *digest)
   return NULL;
 }
 
+/** Allow any network-status newer than this to influence our view of who's
+ * running. */
+#define DEFAULT_RUNNING_INTERVAL 60*60
+/** If possible, always allow at least this many network-statuses to influence
+ * our view of who's running. */
+#define MIN_TO_INFLUENCE_RUNNING 3
+/** Given a list <b>routers</b> of routerinfo_t *, update each routers's
+ * is_named, is_verified, and is_running fields according to our current
+ * networkstatus_t documents. */
+void
+routers_update_status_from_networkstatus(smartlist_t *routers)
+{
+  int n_trusted, n_statuses, n_valid, n_naming, n_named, n_running, n_listing,
+    n_recent;
+  int i;
+  time_t now = time(NULL);
+
+  if (authdir_mode(get_options())) {
+    /* An authoritative directory should never believer someone else about
+     * a server's status. */
+    return;
+  }
+
+  if (!networkstatus_list)
+    networkstatus_list = smartlist_create();
+  if (!trusted_dir_servers)
+    trusted_dir_servers = smartlist_create();
+
+  n_trusted = smartlist_len(trusted_dir_servers);
+  n_statuses = smartlist_len(networkstatus_list);
+
+  if (n_statuses < (n_trusted/2)+1) {
+    /* Not enough statuses to adjust status. */
+    return;
+  }
+
+  if (n_statuses < MIN_TO_INFLUENCE_RUNNING) {
+    n_recent = n_statuses;
+  } else {
+    n_recent = 0;
+    for (i=smartlist_len(networkstatus_list)-1; i >= 0; --i) {
+      networkstatus_t *ns = smartlist_get(networkstatus_list, i);
+      if (ns->published_on + DEFAULT_RUNNING_INTERVAL < now)
+        break;
+      ++n_recent;
+    }
+    if (n_recent < MIN_TO_INFLUENCE_RUNNING) {
+      n_recent = MIN_TO_INFLUENCE_RUNNING;
+    }
+  }
+
+  SMARTLIST_FOREACH(routers, routerinfo_t *, router,
+  {
+    n_listing = n_valid = n_naming = n_named = n_running = 0;
+
+    SMARTLIST_FOREACH(networkstatus_list, networkstatus_t *, ns,
+    {
+      routerstatus_t *rs = networkstatus_find_entry(ns, router->identity_digest);
+      if (ns->binds_names)
+        ++n_naming;
+      if (!rs)
+        continue;
+      ++n_listing;
+      if (rs->is_named && !strcasecmp(rs->nickname, router->nickname))
+        ++n_named;
+      if (rs->is_valid)
+        ++n_valid;
+      if (ns_sl_idx >= smartlist_len(networkstatus_list)-n_recent) {
+        if (rs->is_running)
+          ++n_running;
+      }
+    });
+
+    log_fn(LOG_DEBUG, "Router '%s' at %s:%d is listed by %d/%d directories, "
+           "named by %d/%d, validated by %d/%d, and %d/%d recent directories "
+           "think it's running.",
+           router->nickname, router->address, router->or_port,
+           n_listing, n_statuses, n_named, n_naming, n_valid, n_statuses,
+           n_running, n_recent);
+
+    router->is_named = (n_named > n_naming/2);
+    router->is_verified = (n_valid > n_statuses/2);
+    router->is_running = (n_running > n_recent/2);
+  });
+}
+
+/** Return new list of ID digests for superseded routers.  A router is
+ * superseded if any network-status has a router with a different digest
+ * published more recently, or it it is listed in the network-status but not
+ * in the router list.
+ */
+smartlist_t *
+router_list_superseded(void)
+{
+  smartlist_t *superseded = smartlist_create();
+  strmap_t *most_recent = NULL;
+  char fp[HEX_DIGEST_LEN+1];
+  routerstatus_t *rs_old;
+
+  if (!routerlist || !networkstatus_list)
+    return superseded;
+
+  /* Build a map from fingerprint to most recent routerstatus_t. If this
+   * becomes inefficient, we can build most_recent lazily when a new
+   * networkstatus_t shows up.*/
+  most_recent = strmap_new();
+  SMARTLIST_FOREACH(networkstatus_list, networkstatus_t *, ns,
+  {
+    SMARTLIST_FOREACH(ns->entries, routerstatus_t *, rs,
+    {
+      base16_encode(fp, sizeof(fp), rs->identity_digest, DIGEST_LEN);
+      rs_old = strmap_get(most_recent, fp);
+      if (!rs_old || rs_old->published_on < rs->published_on)
+        strmap_set(most_recent, fp, rs);
+    });
+  });
+
+  /* Compare each router to the most recent routerstatus. */
+  SMARTLIST_FOREACH(routerlist->routers, routerinfo_t *, ri,
+  {
+    routerstatus_t *rs;
+    base16_encode(fp, sizeof(fp), ri->identity_digest, DIGEST_LEN);
+    rs = strmap_get(most_recent, fp);
+    if (!rs)
+      continue;
+    if (memcmp(ri->signed_descriptor_digest,rs->descriptor_digest,DIGEST_LEN)
+        && rs->published_on > ri->published_on) {
+      char *d = tor_malloc(DIGEST_LEN);
+      memcpy(d, ri->identity_digest, DIGEST_LEN);
+      smartlist_add(superseded, d);
+      break;
+    }
+  });
+  strmap_free(most_recent, NULL);
+
+  return superseded;
+}
+
